#include <Configuration.h>
#include <States/GameState/Entities/DynamicEntities/DynamicEntities.h>

Enemy::Enemy() : m_rectangle({config::GAME_ENEMY_SIZE, config::GAME_ENEMY_SIZE}) {
    m_rectangle.setFillColor(config::GAME_ENEMY_COLOR);
    m_rectangle.setOrigin({config::GAME_ENEMY_SIZE/2, config::GAME_ENEMY_SIZE/2});
    m_distance_limit = config::ENEMY_SPEED/config::ROOM_SIZE;
}

std::unique_ptr<IDynamicEntity> Enemy::clone() const {
    return std::make_unique<Enemy>(*this);
}

void Enemy::action() {
    if (m_is_moving) {
        const sf::Vector2f cur_pos = m_rectangle.getPosition();
        sf::Vector2f displacement = m_estimated_position - cur_pos;
        const float distance = std::sqrt(displacement.x * displacement.x + displacement.y * displacement.y);
        if (distance < m_distance_limit) {
            m_rectangle.setPosition(m_estimated_position);
            m_is_moving = false;
            return;
        }
        const float frame_duration = m_stopwatch.getElapsedTime().asSeconds();
        const sf::Vector2f direction = config::ENEMY_SPEED * frame_duration * displacement / distance;
        float delta_x, delta_y;
        if (direction.x > 0)
            delta_x = std::min(direction.x, displacement.x);
        else
            delta_x = std::max(direction.x, displacement.x);
        if (direction.y > 0)
            delta_y = std::min(direction.y, displacement.y);
        else
            delta_y = std::max(direction.y, displacement.y);
        m_rectangle.move(delta_x, delta_y);
    }
    else {
        if (static_cast<size_t>(m_stopwatch.getElapsedTime().asMilliseconds()) < m_dist_milliseconds(m_rng))
            return;
        const auto direction = static_cast<Room::Direction>(m_dist_direction(m_rng));
        m_ptr_room->get_side(direction)->enter(this);
    }
    m_stopwatch.restart();
}

void Enemy::draw_into(sf::RenderWindow& window) const {
    window.draw(m_rectangle);
}

void Enemy::prepare_for_first_drawing() {
    m_rectangle.setPosition(m_ptr_room->get_position());
}

void Enemy::prepare_for_drawing() {
    m_estimated_position = m_ptr_room->get_position();
    m_is_moving = true;
    action();
}
